r"""
***********************
Heritability estimation
***********************

- :func:`.estimate`

Heritability
^^^^^^^^^^^^

.. autofunction:: limix.heritability.estimate

"""

from .estimate import estimate

__all__ = ['estimate']
