# def test_mtSet_simPheno_simulation(script_runner):
#     pass
#     # ret = script_runner.run(bfile='path_to_bfile', cfile='path_to_cfile')
#     # It is not returning anything as of now, but maybe we could return
#     # if it failed or not.
#
#     # Example:
#     # with open('output.txt', 'r') as f:
#     #     r = f.read()
#     #     assert r == 'expected output'
