r"""
*****************************
Variance Decomposition Module
*****************************

- :class:`.VarianceDecomposition`

Public interface
^^^^^^^^^^^^^^^^
"""

from .vardec import VarianceDecomposition

__all__ = ['VarianceDecomposition']
